import gettext
import re
import time
import gi
import os
import platform
import subprocess
import locale
import threading

gi.require_version("Gtk", "4.0")
gi.require_version("Adw", "1")

from gi.repository import Gtk, Adw, Gio, Gdk, GdkPixbuf, GLib, GObject

gettext.install("redoswelcome", "/usr/share/locale")
# gettext.install("redoswelcomegnome", "data/locale")

from locale import gettext as _
from pydbus import SystemBus

locale.bindtextdomain("redoswelcome", "/usr/share/locale")
# locale.bindtextdomain("redoswelcomegnome", "data/locale")

locale.textdomain("redoswelcome")

LAYOUT_STYLE_LEGACY, LAYOUT_STYLE_NEW = range(2)

#devel path
# PARENT_PATH = "/home/wolandius/git_projects/redoswelcome-gnome/data"

# release path
PARENT_PATH = "/usr/share/redos/redoswelcomegnome/"


bus = SystemBus()
the_dbus_starter = bus.get("org.redoswelcome.pydbus.Starter")
whoami = GLib.get_user_name()

from redoswelcomegnome.common import AsyncWorker
# from common import AsyncWorker

class MyException(Exception):
    pass

def set_kwargs(**kwargs):
    app = kwargs.get("app")
    app.rpm = " ".join(rpm for rpm in kwargs.get("rpm_path"))
    app.utility_label = kwargs.get("utility_label")
    app.label_start_text = kwargs.get("label_start_text")
    app.utility_button = kwargs.get("utility_button")
    app.resolve_deps_local()
    app.timeout_id = GLib.timeout_add(60, app.on_timeout, None)


@Gtk.Template.from_file(f"{PARENT_PATH}/ui/templates.ui")
class RedosWelcomeDNF(Adw.ApplicationWindow):
    __gtype_name__ = "rpminstall"
    rpm_install_stack = Gtk.Template.Child()
    resolve_dep_rpm_local = Gtk.Template.Child()
    accept_transaction_rpm = Gtk.Template.Child()
    deny_transaction_rpm = Gtk.Template.Child()
    progressbar = Gtk.Template.Child()
    close_rpm_success = Gtk.Template.Child()
    error_rpm_textview = Gtk.Template.Child()
    close_rpm_error = Gtk.Template.Child()
    error_rpm_textview = Gtk.Template.Child()


    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        #rpm_path, utility_label, label_start_text, utility_button
        css_provider = Gtk.CssProvider()
        css_provider.load_from_file(Gio.File.new_for_path(f"{PARENT_PATH}/ui/style.css"))
        Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(), css_provider,
                                                  Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
        self.rpm_path = None
        self.utility_label = None
        self.label_start_text = None
        self.utility_button = None

    def on_timeout(self, user_data):
        self.progressbar.pulse()
        return True

    def resolve_deps_local(self):
        out = os.popen("systemctl is-active redoswelcome-dbus.service").readline().strip()
        if out == "inactive":
            the_dbus_starter.start()
        self.redos_dnf = bus.get("org.redoswelcome.pydbus.RedosWelcomeDNF", timeout=600)
        if "r7-release" in self.rpm and len(self.rpm) > len("r7-release"):
            result = self.redos_dnf.dnf_command_gpg("r7-release", "/etc/pki/rpm-gpg/RPM-GPG-KEY-R7-OFFICE.public")
            self.rpm = re.sub("r7-release", "", self.rpm)
            if result == "0":
                self.resolve_extra_func()
            else:
                self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
                self.error_rpm_textview.set_text(result)
        elif "yandex-browser-release" in self.rpm and len(self.rpm) > len("yandex-browser-release"):
            result = self.redos_dnf.dnf_command_gpg("yandex-browser-release", "")
            self.rpm = re.sub("yandex-browser-release", "", self.rpm)
            if result == "0":
                self.resolve_extra_func()
            else:
                self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
                self.error_rpm_textview.set_text(result)
        else:
            self.resolve_extra_func()

    def resolve_extra_func(self):
        dependencies, errors = self.redos_dnf.dependencies_local_check(self.rpm).split("|||")
        if "Операция отменена" in errors or "Operation aborted" in errors or "Операция прервана" in errors:
            dependencies = dependencies.split("\n")
            output = ""
            key = -1
            for i in range(0, len(dependencies)):
                if "=" in dependencies[i]:
                    key = i
                    break
            if key > -1:
                dependencies = dependencies[key + 1:]
            dependencies.remove('')
            for i in range(0, len(dependencies)):
                temp = dependencies[i].strip().split()
                print(temp)
                if len(temp) > 5:
                    dependencies[i] = f'\t{temp[0]}-{temp[2]}.{temp[1]} {temp[3]} {temp[4]}{temp[5]}'
                elif len(temp) == 5:
                    dependencies[i] = f'\t{temp[0]}-{temp[2]}.{temp[1]} {temp[3]} {temp[4]}'
                elif len(temp) == 2 and (temp[0] == 'Результат' or temp[0] == 'Transaction'):
                    dependencies[i] = ""
                else:
                    dependencies[i] = f'<b>{dependencies[i]}</b>'
                output += f"{dependencies[i]}\n"
            self.rpm_install_stack.set_visible_child_name("resolve_dep_page_rpm")
            self.resolve_dep_rpm_local.set_markup(output)
            self.resolve_dep_rpm_local.select_region(0, 0)
        elif "Отсутствуют действия для выполнения" in dependencies:
            self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
            dependencies = dependencies.split("\n")
            output = ""
            key = -1
            for i in range(0, len(dependencies)):
                if "=" in dependencies[i]:
                    key = i
                    break
            if key > -1:
                dependencies = dependencies[key + 1:]
            for i in range(0, len(dependencies)):
                output += f"{dependencies[i]}\n"
            self.error_rpm_textview.set_text(output)
        else:
            self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
            self.error_rpm_textview.set_text(errors)

    @Gtk.Template.Callback()
    def on_deny_transaction_rpm_clicked(self, *args):
        self.close()

    @Gtk.Template.Callback()
    def on_close_rpm_success_clicked(self, *args):
        self.close()

    @Gtk.Template.Callback()
    def on_close_rpm_error_clicked(self, *args):
        self.close()

    def close(self):
        self.utility_label.set_text(self.label_start_text)
        self.utility_button.set_sensitive(True)
        if os.path.exists("/tmp/yandex-browser.rpm"):
            os.remove("/tmp/yandex-browser.rpm")
        self.destroy()
    @Gtk.Template.Callback()
    def on_accept_transaction_rpm_clicked(self, *args):
        out = os.popen("systemctl is-active redoswelcome-dbus.service").readline().strip()
        if out == "inactive":
            the_dbus_starter.start()
        self.redos_dnf = bus.get("org.redoswelcome.pydbus.RedosWelcomeDNF", timeout=600)
        granted_access = self.redos_dnf.status_auth()
        if granted_access != "True":
            granted_access = self.redos_dnf.granteAuth(True)
        if granted_access == "rules granted":
            inputs = ()
            async_worker = AsyncWorker(
                operation=self.local_install,
                operation_inputs=inputs,
                operation_callback=self.on_result
            )
            async_worker.start()
        else:
            self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
            self.error_rpm_textview.set_text("Operation not granted")
            the_dbus_starter.stop()


    def local_install(self,  *inputs):
        self.rpm_install_stack.set_visible_child_name("progress_page_rpm_local")
        result = self.redos_dnf.local_install(self.rpm)
        if result == "0":
            os.system("dnf clean all && dnf makecache")
        return result

    def on_result(self, worker, result, handler_data):
        result = worker.return_value(result)
        if result == "0":
            self.rpm_install_stack.set_visible_child_name("success_page_rpm_local")
            the_dbus_starter.stop()
        else:
            self.rpm_install_stack.set_visible_child_name("error_rpm_textview_page")
            self.error_rpm_textview.set_text(result)
            the_dbus_starter.stop()
