#
# RedOS Welcome window utility
# Version: 3.1
# Copyright (c) 2022 Vladlen Murylyov, RED SOFT.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

#!/usr/bin/env python -O
import gettext
import re
import sys
import time
import gi
import os
import platform
import subprocess
import locale
import threading
from redoswelcomegnome import localinstall
from redoswelcomegnome.localinstall import RedosWelcomeDNF
# import localinstall
# from localinstall import RedosWelcomeDNF

from redoswelcomegnome.common import AsyncWorker
# from common import AsyncWorker

gi.require_version("Gtk", "4.0")
gi.require_version("Adw", "1")
from gi.repository import Gtk, Gio, Gdk, GdkPixbuf, GLib, GObject, Adw

NORUN_FLAG = os.path.expanduser("~/.redoswelcome/norun.flag")
GLOBAL_KWARGS = None
gettext.install("redoswelcome", "/usr/share/locale")
# gettext.install("redoswelcome", "data/locale")

from locale import gettext as _

locale.bindtextdomain("redoswelcome", "/usr/share/locale")
# locale.bindtextdomain("redoswelcomegnome", "data/locale")

locale.textdomain("redoswelcome")

LAYOUT_STYLE_LEGACY, LAYOUT_STYLE_NEW = range(2)

VERSION = "3.1"

#devel path
# PARENT_PATH = "/home/wolandius/git_projects/redoswelcome-gnome/data"

#release path
PARENT_PATH = "/usr/share/redos/redoswelcomegnome/"

class MyException(Exception):
    pass

@Gtk.Template.from_file(f"{PARENT_PATH}/ui/redoswelcome.ui")
class RedosWelcome(Adw.ApplicationWindow):
    __gtype_name__ = "MainWindow"

    startup_checkbox = Gtk.Template.Child()
    label_version = Gtk.Template.Child()
    label_edition = Gtk.Template.Child()
    label_welcome_version = Gtk.Template.Child()
    button_light = Gtk.Template.Child()
    button_dark = Gtk.Template.Child()
    image_yandex_browser = Gtk.Template.Child()
    image_r7_doceditors = Gtk.Template.Child()
    image_r7_organizer = Gtk.Template.Child()
    image_r7_graphics = Gtk.Template.Child()

    button_dnfdragora = Gtk.Template.Child()
    button_dnfdragora_upd_all = Gtk.Template.Child()
    button_timeshift = Gtk.Template.Child()
    button_preferred_apps = Gtk.Template.Child()
    button_settings = Gtk.Template.Child()
    button_taskmanager = Gtk.Template.Child()
    button_hardinfo = Gtk.Template.Child()
    button_network = Gtk.Template.Child()
    button_jointodomain = Gtk.Template.Child()
    button_firewall = Gtk.Template.Child()
    button_officeopen = Gtk.Template.Child()
    button_printers = Gtk.Template.Child()

    button_yandex_browser = Gtk.Template.Child()
    button_r7_doceditors = Gtk.Template.Child()
    button_r7_organizer = Gtk.Template.Child()
    button_r7_graphics = Gtk.Template.Child()

    button_redos_changelogs = Gtk.Template.Child()
    button_aboutos = Gtk.Template.Child()

    yandex_label = Gtk.Template.Child()
    r7_doceditors_label = Gtk.Template.Child()
    r7_organizer_label = Gtk.Template.Child()
    r7_graphics_label = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        css_provider = Gtk.CssProvider()
        css_provider.load_from_file(Gio.File.new_for_path(f"{PARENT_PATH}/ui/style.css"))
        Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(), css_provider,
                                                  Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

        # self.startup_checkbox.set_sensitive(True)

        # self.startup_checkbox.set_label(_("Show this dialog at startup"))

        self.startup_checkbox.set_active(False) if os.path.exists(NORUN_FLAG) else self.startup_checkbox.set_active(True)

        architecture = "64-bit"
        if platform.machine() != "x86_64":
            architecture = str(platform.machine())
        dist_name = "РЕД ОС"
        release = re.sub("Release:", "", os.popen("lsb_release -r").readline()).strip()
        if "n/a" in release:
            release = "8"
        elif "8.0" in release:
            release = "8"
        self.description = "standart" if "standard" in os.popen("lsb_release -d").readline().lower() else "certified"
        # description = "certified"
        edition = "Unknown DE or in Root mode"
        if os.getenv("XDG_CURRENT_DESKTOP") == "X-Cinnamon" or os.getenv("XDG_CURRENT_DESKTOP") == "Cinnamon":
            edition = "Cinnamon"
        elif os.getenv("XDG_CURRENT_DESKTOP") == "MATE":
            edition = "Mate"
        elif os.getenv("XDG_CURRENT_DESKTOP") == "GNOME":
            edition = "GNOME"
        elif os.getenv("XDG_CURRENT_DESKTOP") == "KDE":
            edition = "KDE"
        self.label_version.set_markup("<b>%s %s</b>" % (dist_name, release))
        self.label_edition.set_text("%s %s" % (edition, architecture))
        self.label_welcome_version.set_text(f"v{VERSION}")
        self.GLOBAL_KWARGS = None


        # self.image_youtube.set_from_file(
        #     "/home/wolandius/git_projects/redoswelcome-gnome/data/ui/youtube.png")


        # self.baseimage.set_from_file("/home/wolandius/Загрузки/РЕД_ОС.svg")

    @Gtk.Template.Callback()
    def on_button_toggled(self, *args):
        button = args[0]
        if button.get_active():
            if os.path.exists(NORUN_FLAG):
                os.system("rm -rf %s" % NORUN_FLAG)
        else:
            os.system("mkdir -p ~/.redoswelcome")
            os.system("touch %s" % NORUN_FLAG)
    @Gtk.Template.Callback()
    def on_button_light_clicked(self, *args):
        self.call_change_theme_color_gnome("prefer-light")

    @Gtk.Template.Callback()
    def on_button_dark_clicked(self, *args):
        self.call_change_theme_color_gnome("prefer-dark")

    @Gtk.Template.Callback()
    def on_button_dnfdragora_clicked(self, *args):
        self.launch("dnfdragora")

    @Gtk.Template.Callback()
    def on_button_dnfdragora_upd_all_clicked(self, *args):
        self.launch_args(["dnfdragora", "--update-only"])

    @Gtk.Template.Callback()
    def on_button_timeshift_clicked(self, *args):
        self.launch("timeshift-launcher")

    @Gtk.Template.Callback()
    def on_button_preferred_apps_clicked(self, *args):
        self.launch_args(["gnome-control-center", "default-apps"])

    @Gtk.Template.Callback()
    def on_button_settings_clicked(self, *args):
        self.launch("gnome-control-center")

    @Gtk.Template.Callback()
    def on_button_taskmanager_clicked(self, *args):
        self.launch("gnome-system-monitor")

    @Gtk.Template.Callback()
    def on_button_hardinfo_clicked(self, *args):
        self.launch_args(["gnome-control-center", "info-overview"])
    @Gtk.Template.Callback()
    def on_button_network_clicked(self, *args):
        self.launch_args(["gnome-control-center", "network"])
    @Gtk.Template.Callback()
    def on_button_jointodomain_clicked(self, *args):
        self.launch_args(["pkexec", "join-to-domain.sh", "-g"])
    @Gtk.Template.Callback()
    def on_button_firewall_clicked(self, *args):
        self.launch("firewall-config")
    @Gtk.Template.Callback()
    def on_button_officeopen_clicked(self, *args):
        self.launch("libreoffice")
    @Gtk.Template.Callback()
    def on_button_printers_clicked(self, *args):
        self.launch_args(["gnome-control-center", "printers"])

    @Gtk.Template.Callback()
    def on_button_yandex_browser_clicked(self, *args):
        self.returncode = None
        self.label_start_text = _(
            "Install Yandex.Browser, a safe and fast Russian browser that supports TLS protocols according to GOST. It automatically translates videos, stores your passwords securely, and protects against viruses and scammers.")
        self.button_yandex_browser.set_sensitive(False)
        self.yandex_label.set_text(_("Preparing for installation"))

        inputs = ()
        async_worker = AsyncWorker(
            operation=self.download_yandex,
            operation_inputs=inputs,
            operation_callback=self.start_yandex_install
        )
        async_worker.start()

    @Gtk.Template.Callback()
    def on_button_redos_changelogs_clicked(self, *args):
        self.visit("https://redos.red-soft.ru/go/changelog/8-0-latest/") if "standart" in self.description else \
                self.visit("https://redos.red-soft.ru/go/changelog/8-0c-latest/")

    def visit(self, url):
        subprocess.Popen(["xdg-open", url])

    @Gtk.Template.Callback()
    def on_button_aboutos_clicked(self, *args):
        self.visit("https://redos.red-soft.ru/")


    def call_change_theme_color_gnome(self, color):
        settings = Gio.Settings(schema="org.gnome.desktop.interface")
        settings.set_string("color-scheme", color)

    def launch(self, command):
        if "dnfdragora" in command:
            self.kill_dnfdragora()
        subprocess.Popen(command)

    def launch_args(self, command):
        for arg in command:
            if "dnfdragora" in arg:
                self.kill_dnfdragora()
                break
        subprocess.Popen(command)

    def pkexec(self, command):
        subprocess.Popen(["pkexec", f"{command}"])

    def kill_dnfdragora(self):
        os.system("kill -9 $(ps aux | grep '[d]nfdragora --\|[d]nfdragora$' | awk '{print $2}')")



    def download_yandex(self, *inputs):
        check_install = os.popen("rpm -qa | grep yandex-browser-stable").readlines()
        check_repo_file = os.popen("rpm -qa | grep yandex-browser-release").readlines()
        if len(check_repo_file) == 0:
            if len(check_install) == 0:
                # wget_out = subprocess.Popen(["wget", "https://redos.red-soft.ru/go/yandex-browser",
                #                              "-O", "/tmp/yandex-browser.rpm"],
                #                             stdout=subprocess.PIPE,
                #                             stderr=subprocess.PIPE)
                # output, error = wget_out.communicate()
                # if wget_out.returncode != 0:
                #     self.yandex_label.set_text(error.decode("utf-8").strip())
                #     time.sleep(3)
                #     self.yandex_label.set_text(self.label_start_text)
                #     self.button_yandex_browser.set_sensitive(True)
                #     return 1
                # else:
                return "ya-all"
            else:
                return "ya-release"
        else:
            if len(check_install) == 0:
                return "ya-browser"
            else:
                self.yandex_label.set_text(_("Package already installed"))
                return 1
    def on_activate_rpm(self, application, **kwargs):
        win = RedosWelcomeDNF(application=application)
        win.set_default_size(800, 600)
        localinstall.set_kwargs(app=win, rpm_path=self.GLOBAL_KWARGS['rpm_path'], utility_label=self.GLOBAL_KWARGS['utility_label'], label_start_text=self.GLOBAL_KWARGS['label_start_text'], utility_button=self.GLOBAL_KWARGS['utility_button'])
        win.present()

    def start_yandex_install(self, worker, result, handler_data):
        result = worker.return_value(result)
        if result == "ya-all":
            self.GLOBAL_KWARGS = {'rpm_path': ["yandex-browser-stable", "yandex-browser-release"],
                             'utility_label': self.yandex_label, 'label_start_text': self.label_start_text,
                             'utility_button': self.button_yandex_browser}
            self.run_adw()
        elif result == "ya-release":
            self.GLOBAL_KWARGS = {'rpm_path': ["yandex-browser-release"],
                             'utility_label': self.yandex_label, 'label_start_text': self.label_start_text,
                             'utility_button': self.button_yandex_browser}
            self.run_adw()

        elif result == "ya-browser":
            self.GLOBAL_KWARGS = {'rpm_path': ["yandex-browser-stable"],
                             'utility_label': self.yandex_label, 'label_start_text': self.label_start_text,
                             'utility_button': self.button_yandex_browser}
            self.run_adw()

        elif result == 1:
            time.sleep(3)
            self.yandex_label.set_text(self.label_start_text)
            self.button_yandex_browser.set_sensitive(True)

    @Gtk.Template.Callback()
    def on_button_r7_doceditors_clicked(self, *args):
        self.returncode = None
        self.label_start_text = _(
            "Professional document, spreadsheet and presentation editors.")
        self.button_r7_doceditors.set_sensitive(False)
        self.r7_doceditors_label.set_text(_("Preparing for installation"))

        inputs = ()
        async_worker = AsyncWorker(
            operation=self.download_r7_doceditors,
            operation_inputs=inputs,
            operation_callback=self.start_r7_install
        )
        async_worker.start()

    @Gtk.Template.Callback()
    def on_button_r7_graphics_clicked(self, *args):
        self.returncode = None
        self.label_start_text = _(
            "A tool for creating professional-grade diagrams, diagrams, graphs and business illustrations.")
        self.button_r7_graphics.set_sensitive(False)
        self.r7_graphics_label.set_text(_("Preparing for installation"))

        inputs = ()
        async_worker = AsyncWorker(
            operation=self.download_r7_graphics,
            operation_inputs=inputs,
            operation_callback=self.start_r7_install
        )
        async_worker.start()

    @Gtk.Template.Callback()
    def on_button_r7_organizer_clicked(self, *args):
        self.returncode = None
        self.label_start_text = _(
            "Application for managing mail, contacts and business schedule.")
        self.button_r7_organizer.set_sensitive(False)
        self.r7_organizer_label.set_text(_("Preparing for installation"))
        inputs = ()
        async_worker = AsyncWorker(
            operation=self.download_r7_organizer,
            operation_inputs=inputs,
            operation_callback=self.start_r7_install
        )
        async_worker.start()

    def download_r7_doceditors(self, *inputs):
        check_install = os.popen("rpm -qa | grep r7-office").readlines()
        check_repo_file = os.popen("rpm -qa | grep r7-release").readlines()
        if len(check_repo_file) == 0:
            if len(check_install) == 0:
                return "allr7o"
            else:
                return "orelease"
        else:
            if len(check_install) == 0:
                return "r7o"
            else:
                self.r7_doceditors_label.set_text(_("Package already installed"))
                return "1o"

    def download_r7_organizer(self, *inputs):
        check_install = os.popen("rpm -qa | grep r7organizer").readlines()
        check_repo_file = os.popen("rpm -qa | grep r7-release").readlines()
        if len(check_repo_file) == 0:
            if len(check_install) == 0:
                return "allr7org"
            else:
                return "orgrelease"
        else:
            if len(check_install) == 0:
                return "r7org"
            else:
                self.r7_organizer_label.set_text(_("Rpm already installed"))
                return "1org"

    def download_r7_graphics(self, *inputs):
        check_install = os.popen("rpm -qa | grep R7Grafika").readlines()
        check_repo_file = os.popen("rpm -qa | grep r7-release").readlines()
        if len(check_repo_file) == 0:
            if len(check_install) == 0:
                return "allr7g"
            else:
                return "grelease"
        else:
            if len(check_install) == 0:
                return "r7g"
            else:
                self.r7_graphics_label.set_text(_("Rpm already installed"))
                return "1g"


    def run_adw(self):
        app_rpm = Adw.Application(application_id="org.redos.redoswelcome.rpminstall")
        app_rpm.connect("activate", self.on_activate_rpm)
        app_rpm.run([])


    def start_r7_install(self, worker, result, handler_data):
        """
        allr7o - means r7 repo +  r7-office
        allr7g - means r7 repo +  R7Grafika
        allr7org - means r7 repo +  r7organizer
        r7o - r7-office
        r7g - r7-R7Grafika
        r7org - r7organizer
        """
        result = worker.return_value(result)
        if result == "allr7o":
            self.GLOBAL_KWARGS = {'rpm_path': ["r7-office", "r7-release"],
                                  'utility_label': self.r7_doceditors_label, 'label_start_text': self.label_start_text,
                                  'utility_button': self.button_r7_doceditors}
            self.run_adw()
        elif result == "allr7g":
            self.GLOBAL_KWARGS = {'rpm_path': ["R7Grafika", "r7-release"],
                                  'utility_label': self.r7_graphics_label, 'label_start_text': self.label_start_text,
                                  'utility_button': self.button_r7_graphics}
            self.run_adw()
        elif result == "allr7org":
            self.GLOBAL_KWARGS = {'rpm_path': ["r7organizer", "r7-release"],
                                  'utility_label': self.r7_organizer_label, 'label_start_text': self.label_start_text,
                                  'utility_button': self.button_r7_organizer}
            self.run_adw()
        elif result == "orelease":
            self.GLOBAL_KWARGS = {'rpm_path': ["r7-release"],
                                  'utility_label': self.r7_doceditors_label, 'label_start_text': self.label_start_text,
                                  'utility_button': self.button_r7_doceditors}
            self.run_adw()
        elif result == "orgrelease":
            self.GLOBAL_KWARGS = {'rpm_path': ["r7-release"],
                                  'utility_label': self.r7_organizer_label, 'label_start_text': self.label_start_text,
                                  'utility_button': self.button_r7_organizer}
            self.run_adw()
        elif result == "grelease":
            self.GLOBAL_KWARGS = {'rpm_path': ["r7-release"],
                                  'utility_label': self.r7_graphics_label, 'label_start_text': self.label_start_text,
                                  'utility_button': self.button_r7_graphics}
            self.run_adw()
        elif result == "r7o":
            self.GLOBAL_KWARGS = {'rpm_path': ["r7-office"],
                                  'utility_label': self.r7_doceditors_label, 'label_start_text': self.label_start_text,
                                  'utility_button': self.button_r7_doceditors}
            self.run_adw()
        elif result == "r7g":
            self.GLOBAL_KWARGS = {'rpm_path': ["R7Grafika"],
                                  'utility_label': self.r7_graphics_label, 'label_start_text': self.label_start_text,
                                  'utility_button': self.button_r7_graphics}
            self.run_adw()
        elif result == "r7org":
            self.GLOBAL_KWARGS = {'rpm_path': ["r7organizer"],
                                  'utility_label': self.r7_organizer_label, 'label_start_text': self.label_start_text,
                                  'utility_button': self.button_r7_organizer}
            self.run_adw()
        elif result == "1o":
            time.sleep(3)
            self.r7_doceditors_label.set_text(self.label_start_text)
            self.button_r7_doceditors.set_sensitive(True)
        elif result == "1org":
            time.sleep(3)
            self.r7_organizer_label.set_text(self.label_start_text)
            self.button_r7_organizer.set_sensitive(True)
        elif result == "1g":
            time.sleep(3)
            self.r7_graphics_label.set_text(self.label_start_text)
            self.button_r7_graphics.set_sensitive(True)


def on_activate(application):
    win = RedosWelcome(application=application)
    # win.set_icon_name('system-logo-icon')
    win.set_default_size(800, 600)
    win.present()


def main():
    app = Adw.Application(application_id="org.redos.redoswelcome")
    app.connect("activate", on_activate)
    app.run([])

if __name__ == "__main__":
    main()
